package de.berlin.airport.artemis;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map.Entry;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MalformedObjectNameException;
import javax.management.ReflectionException;
import javax.naming.NamingException;

import org.apache.qpid.protonj2.client.Client;
import org.apache.qpid.protonj2.client.Connection;
import org.apache.qpid.protonj2.client.ConnectionOptions;
import org.apache.qpid.protonj2.client.Delivery;
import org.apache.qpid.protonj2.client.Message;
import org.apache.qpid.protonj2.client.Receiver;
import org.apache.qpid.protonj2.client.Sender;
import org.apache.qpid.protonj2.client.exceptions.ClientException;

import de.berlin.airport.artemis.json.Connections;
import de.berlin.airport.artemis.json.Consumer;
import de.berlin.airport.artemis.json.Consumers;
import de.berlin.airport.artemis.json.Producer;
import de.berlin.airport.artemis.json.Producers;
import de.berlin.airport.artemis.json.Queue;
import de.berlin.airport.artemis.json.QueuePage;
import de.berlin.airport.artemis.json.Session;
import de.berlin.airport.artemis.json.Sessions;

public class Main {

    private static String QUEUE = "foobar";
    private static String USERNAME = "artemis";
    private static String PASSWORD = "artemis";
    private static String AMQP_HOST = "localhost";
    private static String BROKER_NAME = "0.0.0.0";
    private static int AMQP_PORT = 5672;
    private static String JMX_HOST = "localhost";
    private static int JMX_PORT = 1099;

    public void run() throws IOException, MalformedObjectNameException, InstanceNotFoundException, MBeanException,
            ReflectionException, NamingException, ClientException {
        JMXBroker jmxBroker = new JMXBroker(BROKER_NAME);
        jmxBroker.connect(JMX_HOST, JMX_PORT, USERNAME, PASSWORD);
        // JMXQueue jmxQueue = jmxBroker.getQueueJMX(QUEUE);

        ConnectionOptions ampqConnectionOptions = new ConnectionOptions();
        ampqConnectionOptions.user(USERNAME);
        ampqConnectionOptions.password(PASSWORD);

        Client amqpClientSender = Client.create();
        Connection amqpConnectionSender = amqpClientSender.connect(AMQP_HOST, AMQP_PORT, ampqConnectionOptions);
        Sender amqpSender = amqpConnectionSender.openSender(QUEUE);

        Client amqpClientReceiver = Client.create();
        Connection amqpConnectionReceiver = amqpClientReceiver.connect(AMQP_HOST, AMQP_PORT,
                ampqConnectionOptions);
        Receiver amqpReceiver = amqpConnectionReceiver.openReceiver(QUEUE);

        amqpSender.send(Message.create("Hello baz"));
        Delivery amqpDelivery = amqpReceiver.receive();
        System.out.println(amqpDelivery.message().body().toString());

        Connections connections = jmxBroker.getConnections();
        Sessions sessions = jmxBroker.getSessions();
        QueuePage queuePage = jmxBroker.getQueuesPaged(
                "{\"field\":\"\",\"operation\":\"\",\"value\":\"\",\"sortOrder\":\"asc\",\"sortColumn\":\"name\"}",
                1,
                200);
        Producers producers = jmxBroker.getProducers();
        HashMap<de.berlin.airport.artemis.json.Connection, Consumers> consumersPerConnection = new HashMap<>();
        for (de.berlin.airport.artemis.json.Connection connection : connections) {
            consumersPerConnection.put(connection, jmxBroker.getConsumers(connection.getConnectionID()));
        }

        System.out.println(".==================================================.");
        System.out.println("| Connections                                      |");
        System.out.println("|==================================================|");
        System.out.println("| Connection ID | Client Address       | #Sessions |");
        System.out.println("|---------------|----------------------|-----------|");
        for (de.berlin.airport.artemis.json.Connection connection : connections) {
            System.out.println(String.format("| %-13s | %-20s | %9d |", connection.getConnectionID(),
                    connection.getClientAddress(), connection.getSessionCount()));
        }
        System.out.println("'=================================================='");

        System.out.println(".===================================================================.");
        System.out.println("| Sessions                                                          |");
        System.out.println("|===================================================================|");
        System.out.println("| Session ID                           | #Consumer | User           |");
        System.out.println("|--------------------------------------|-----------|----------------|");
        for (Session session : sessions) {
            System.out.println(String.format("| %-36s | %9d | %-14s |", session.getSessionID(),
                    session.getConsumerCount(), session.getValidatedUser()));
        }
        System.out.println("'==================================================================='");

        System.out.println(
                ".=====================================================================================================.");
        System.out.println(
                "| Queues                                                                                              |");
        System.out.println(
                "|=====================================================================================================|");
        System.out.println(
                "| Queue ID | Address                    | Queue Name                           | #Consumer | #Message |");
        System.out.println(
                "|----------|----------------------------|--------------------------------------|-----------|----------|");
        for (Queue queue : queuePage.getData()) {
            System.out.println(String.format("| %8d | %-26s | %-36s | %9d | %8d |", queue.getId(),
                    queue.getAddress(),
                    queue.getName(), queue.getConsumerCount(), queue.getMessageCount()));
        }
        System.out.println(
                "'====================================================================================================='");

        System.out.println(
                ".================================================================================================================.");
        System.out.println(
                "| Producers                                                                                                      |");
        System.out.println(
                "|================================================================================================================|");
        System.out.println(
                "| Producer ID | Connection ID | Session ID                           | Destination                | #MessageSent |");
        System.out.println(
                "|-------------|---------------|--------------------------------------|----------------------------|--------------|");
        for (Producer producer : producers) {
            System.out.println(String.format("| %11d | %-13s | %-36s | %-26s | %12d |", producer.getId(),
                    producer.getConnectionID(), producer.getSessionID(), producer.getDestination(),
                    producer.getMsgSent()));
        }
        System.out.println(
                "'================================================================================================================'");

        System.out.println(
                ".==========================================================================================================================================================.");
        System.out.println(
                "| Consumers                                                                                                                                                |");
        System.out.println(
                "|==========================================================================================================================================================|");
        System.out.println(
                "| Connection ID | Consumer ID | Session ID                           | Queue Name                           | #MessagesDelivered | LastDeliveredTime       |");
        System.out.println(
                "|---------------|-------------|--------------------------------------|--------------------------------------|--------------------|-------------------------|");
        for (Entry<de.berlin.airport.artemis.json.Connection, Consumers> consumers : consumersPerConnection
                .entrySet()) {
            for (Consumer consumer : consumers.getValue()) {
                System.out.println(String.format("| %-13s | %11d | %-36s | %-36s | %18d | %-23s |",
                        consumer.getConnectionID(),
                        consumer.getConsumerID(),
                        consumer.getSessionID(),
                        consumer.getQueueName(),
                        consumer.getMessagesDelivered(),
                        consumer.getLastDeliveredTimeLocal()));
            }
        }
        System.out.println(
                "'=========================================================================================================================================================='");

        // System.in.read();

        jmxBroker.disconnect();
        amqpConnectionSender.close();
        amqpConnectionReceiver.close();
    }

    public static void main(String[] args) {
        try {
            if (args.length == 8) {
                int i = 0;
                Main.AMQP_HOST = args[i++];
                Main.AMQP_PORT = Integer.parseInt(args[i++]);
                Main.JMX_HOST = args[i++];
                Main.JMX_PORT = Integer.parseInt(args[i++]);
                Main.USERNAME = args[i++];
                Main.PASSWORD = args[i++];
                Main.BROKER_NAME = args[i++];
                Main.QUEUE = args[i++];
            } else {
                System.err.println("Usage: Main <amqp host> <amqp port> <jmx host> <jmx port> <username> <password> <broker name> <queue>");
                System.exit(1);
            }

            Main main = new Main();
            main.run();
        } catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace(System.err);
        }
    }
}